<?php

namespace Litecms\Register\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Register\Http\Requests\RegisterRequest;
use Litecms\Register\Interfaces\RegisterRepositoryInterface;
use Litecms\Register\Models\Register;

/**
 * Resource controller class for register.
 */
class RegisterResourceController extends BaseController
{

    /**
     * Initialize register resource controller.
     *
     * @param type RegisterRepositoryInterface $register
     *
     * @return null
     */
    public function __construct(RegisterRepositoryInterface $register)
    {
        parent::__construct();
        $this->repository = $register;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Register\Repositories\Criteria\RegisterResourceCriteria::class);
    }

    /**
     * Display a list of register.
     *
     * @return Response
     */
    public function index(RegisterRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Register\Repositories\Presenter\RegisterPresenter::class)
                ->$function();
        }

        $registers = $this->repository->paginate();

        return $this->response->title(trans('register::register.names'))
            ->view('register::register.index', true)
            ->data(compact('registers'))
            ->output();
    }

    /**
     * Display register.
     *
     * @param Request $request
     * @param Model   $register
     *
     * @return Response
     */
    public function show(RegisterRequest $request, Register $register)
    {

        if ($register->exists) {
            $view = 'register::register.show';
        } else {
            $view = 'register::register.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('register::register.name'))
            ->data(compact('register'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new register.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RegisterRequest $request)
    {

        $register = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('register::register.name')) 
            ->view('register::register.create', true) 
            ->data(compact('register'))
            ->output();
    }

    /**
     * Create new register.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RegisterRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $register                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('register::register.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('register/register/' . $register->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/register/register'))
                ->redirect();
        }

    }

    /**
     * Show register for editing.
     *
     * @param Request $request
     * @param Model   $register
     *
     * @return Response
     */
    public function edit(RegisterRequest $request, Register $register)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('register::register.name'))
            ->view('register::register.edit', true)
            ->data(compact('register'))
            ->output();
    }

    /**
     * Update the register.
     *
     * @param Request $request
     * @param Model   $register
     *
     * @return Response
     */
    public function update(RegisterRequest $request, Register $register)
    {
        try {
            $attributes = $request->all();

            $register->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('register::register.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('register/register/' . $register->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('register/register/' . $register->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the register.
     *
     * @param Model   $register
     *
     * @return Response
     */
    public function destroy(RegisterRequest $request, Register $register)
    {
        try {

            $register->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('register::register.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('register/register/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('register/register/' . $register->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple register.
     *
     * @param Model   $register
     *
     * @return Response
     */
    public function delete(RegisterRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('register::register.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('register/register'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/register/register'))
                ->redirect();
        }

    }

    /**
     * Restore deleted registers.
     *
     * @param Model   $register
     *
     * @return Response
     */
    public function restore(RegisterRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('register::register.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/register/register'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/register/register/'))
                ->redirect();
        }

    }

}
